# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-31 06:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.UUIDField(editable=False, primary_key=True, serialize=False)),
                ('status', models.CharField(choices=[('enqueued', 'Enqueued'), ('delayed', 'Delayed'), ('running', 'Running'), ('failed', 'Failed'), ('done', 'Done')], default='enqueued', max_length=8)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('message_data', models.BinaryField()),
            ],
            options={
                'ordering': ['-updated_at'],
            },
            managers=[
                ('tasks', django.db.models.manager.Manager()),
            ],
        ),
    ]
